Imports System.Drawing.Drawing2D
Imports System.ComponentModel
''' <summary>
''' ein Control zur Anzeige und Auswahl von Tagen in einem Monat
''' </summary>
Public Class MonthControl
    Inherits Control

#Region " Private Fields / Events "
    ' Auflistung mit enthaltenen Tages-Controls
    Private m_DayControls As New List(Of DayControl)
    ' aktuell selektiertes Tages-Control
    Private m_SelectedDayControl As DayControl

    ' Referenzierte Liste mit enthaltenen Zeitbereichen
    Private m_BasicList As List(Of TimeArea) = Nothing

    ' aktulell angezeigtes Jahr
    Private m_Year As Integer
    ' aktuell angezeigter Monat
    Private m_Month As Integer
    ' erstes Datum, fr das auf diesem Control ein Tag angezeigt wird
    Private m_StartDatum As Date

    ' Hintergrund-Rechteck
    Private m_BackRect As RectangleF
    ' Hintergrund-Brush
    Private m_BackBrush As LinearGradientBrush

    ''' <summary>
    ''' Event bei nderung der Tages-Auswahl
    ''' </summary>
    Public Event Changed()
#End Region

#Region " Properties "
    ''' <summary>
    ''' Gibt die Auflistung mit Zeitbereichen, deren Inhalt angezeigt werden soll, zurck oder 
    ''' legt diese fest
    ''' </summary>
    <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
    Property TimeAreas() As List(Of TimeArea)
        Get
            Return m_BasicList
        End Get
        Set(ByVal value As List(Of TimeArea))
            If m_BasicList IsNot value Then
                If m_BasicList IsNot Nothing Then
                    ' eventuell vernderte Inhalte schreiben
                    Save_Values()
                End If
                ' Liste bernehmen
                m_BasicList = value
                ' anzuzeigende Inhalte lesen
                CreateDayControls()
            End If
        End Set
    End Property

    ''' <summary>
    ''' das fr die Monats-Anzeige verwendete Basis-Datum
    ''' </summary>
    ''' <remarks>kann ein beliebiges Datum in einem Monat sein</remarks>
    <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
    WriteOnly Property Datum() As Date
        Set(ByVal value As Date)
            If m_Year <> value.Year OrElse m_Month <> value.Month Then
                If m_BasicList IsNot Nothing Then
                    ' eventuell vernderte Inhalte schreiben
                    Save_Values()
                End If
                ' verwendete Datums-Informationen aktualisieren
                ChangeBasicDates(value.Year, value.Month)
                ' anzuzeigende Inhalte lesen
                CreateDayControls()
            End If
        End Set
    End Property

    ''' <summary>
    ''' gibt die Zeitbereichsliste fr das aktuell ausgewhlte Tages-Control zurck
    ''' </summary>
    ReadOnly Property SelectedZeitBereich() As TimeAreaList
        Get
            Return m_SelectedDayControl.DayTimeAreas
        End Get
    End Property

    ''' <summary>
    ''' gibt das Datum fr das aktuell ausgewhlte Tages-Control zurck
    ''' </summary>
    ReadOnly Property SelectedDate() As Date
        Get
            Return m_SelectedDayControl.DayTimeAreas.Datum
        End Get
    End Property
#End Region

#Region " Constructor "
    Public Sub New()
        ' DoubleBuffer aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint Or ControlStyles.Selectable, True)

        ' angezeigte Datumsangaben ermitteln
        Dim StartDate As Date = Date.Today
        ChangeBasicDates(StartDate.Year, StartDate.Month)
        ' Tages-Controls erstellen
        CreateDayControls()
    End Sub
#End Region

#Region " Methods "
    ''' <summary>
    ''' ermittelt intern bentigte Datums-Informationen
    ''' </summary>
    Private Sub ChangeBasicDates(ByVal Year As Integer, ByVal Month As Integer)
        m_Year = Year
        m_Month = Month

        Dim BasisDatum As New Date(m_Year, m_Month, 1)
        Dim Weekday As Integer = CInt(BasisDatum.DayOfWeek) - 1
        If Weekday = -1 Then
            Weekday = 6
        End If

        ' erstes Datum, fr das auf diesem Control ein Tag angezeigt wird
        m_StartDatum = BasisDatum.AddDays(-Weekday)
    End Sub

    ''' <summary>
    ''' speichert die aktuell angezeigten Datums-Informtionen
    ''' zurck in die Basis-Auflistung
    ''' </summary>
    ''' <remarks>passiert beim ndern von Anzeige-Datum oder Basis-Liste,
    ''' muss aber beim Schlieen der Windows Form / des Parent-Controls 
    ''' extra nochmals ausgefhrt werden</remarks>
    Public Sub Save_Values()
        ' angezeigtes End-Datum
        Dim EndDate As Date = m_StartDatum.AddDays(41)
        ' alle Zeitbereiche, die zwischen Start- und Enddatum liegen, aus der Basis-Liste entfernen 
        For I As Integer = m_BasicList.Count - 1 To 0 Step -1
            If m_BasicList(I).Start.Date >= m_StartDatum AndAlso m_BasicList(I).Ende.Date <= EndDate Then
                m_BasicList.RemoveAt(I)
            End If
        Next
        ' die aktuell angezeigten Zeitbereiche bernehmen
        For Each Ctl As DayControl In m_DayControls
            For Each Bereich As TimeArea In Ctl.DayTimeAreas
                m_BasicList.Add(Bereich)
            Next
        Next
    End Sub

    ''' <summary>
    ''' erstellt die angezeigten Tages-Controls neu
    ''' </summary>
    ''' <remarks>jedes Control enthlt eine eigene Tages-Zeitbereichsliste</remarks>
    Private Sub CreateDayControls()

        ' Temporre Liste mit angezeigten Zeitbereichen
        Dim TmpListe As New List(Of TimeArea)

        If m_BasicList IsNot Nothing Then
            ' wenn eine Basis-Liste vorhanden ist, hier nach Zeitbereichen,
            ' die zur aktuellen Anzeige passen, suchen
            Dim EndDate As Date = m_StartDatum.AddDays(41)
            For Each Bereich As TimeArea In m_BasicList
                If Bereich.Start.Date >= m_StartDatum AndAlso Bereich.Ende.Date <= EndDate Then
                    TmpListe.Add(Bereich)
                End If
            Next
        End If

        ' Eventhandler fr zu entfernende Tages-Controls entfernen
        For Each Ctl As DayControl In m_DayControls
            RemoveHandler Ctl.WasSelected, AddressOf DayControl_WasSelected
        Next
        ' Liste mit Tages-Controls entleeren
        m_DayControls.Clear()

        Dim Left, Top As Integer
        Top = 5
        Left = 5

        ' Fr alle anzuzeigenden Tage
        For I As Integer = 0 To 41
            ' neues Tages-Control erstellen
            Dim Ctl As New DayControl(New DayTimeAreaList(m_StartDatum.AddDays(I), TmpListe))
            Ctl.Location = New Point(Left, Top)
            If Ctl.DayTimeAreas.Datum.DayOfWeek = DayOfWeek.Sunday Then
                Left = 5
                Top += 55
            Else
                Left += 105
            End If
            If Ctl.DayTimeAreas.Datum.Month <> m_Month Then
                Ctl.OutOfMonth = True
            End If
            ' Control zur auflistung mit Tages-Controls hinzufgen
            m_DayControls.Add(Ctl)
            ' Eventhandler fr das Selected-Event hinzufgen
            AddHandler Ctl.WasSelected, AddressOf DayControl_WasSelected
        Next

        ' Layout-Logik unterbrechen
        Me.SuspendLayout()
        ' bisher vorhandene Controls entfernen
        Me.Controls.Clear()
        ' neue Tages-Controls zur eigenen Control-Auflistung hinzufgen
        Me.Controls.AddRange(m_DayControls.ToArray)

        ' Index fr aktuellen Monats-Start suchen
        Dim Idx As Integer
        For Each Ctl As DayControl In m_DayControls
            If Ctl.DayTimeAreas.Datum.Month = m_Month Then
                Idx = m_DayControls.IndexOf(Ctl)
                Exit For
            End If
        Next
        ' das zugehrige Control selektieren
        m_DayControls(Idx).Selected = True
        m_SelectedDayControl = m_DayControls(Idx)
        RaiseEvent Changed()

        ' Layout-Logik wieder aufnehmen
        Me.ResumeLayout()
    End Sub

    ''' <summary>
    ''' Handler fr das WasSelected-Event der Tages-Controls
    ''' </summary>
    Private Sub DayControl_WasSelected(ByVal Ctl As DayControl)
        ' mehrfache Aufrufe unterbinden
        If m_SelectedDayControl Is Ctl Then Exit Sub
        ' alle brigen Controls deselektieren
        For Each xCtl As DayControl In m_DayControls
            If Not xCtl Is Ctl Then
                xCtl.Selected = False
            End If
        Next
        ' In der Folge Changed-Event fr das Monats-Control auslsen
        m_SelectedDayControl = Ctl
        RaiseEvent Changed()
    End Sub

    ''' <summary>
    ''' berschreiben von OnPaint, um zustzliche Inhalte dieses Controls zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)
        e.Graphics.FillRectangle(m_BackBrush, m_BackRect)
        ControlPaint.DrawVisualStyleBorder(e.Graphics, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub

    ''' <summary>
    ''' berschreiben von OnResize, um grenabhngige Objekte neu zu definieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal eventargs As System.EventArgs)
        MyBase.OnResize(eventargs)
        m_BackRect = New RectangleF(0, 0, Me.Width, Me.Height)
        m_BackBrush = New LinearGradientBrush(m_BackRect, Color.WhiteSmoke, Color.LightGray, 45.0F)
    End Sub

    ''' <summary>
    ''' berschreiben von OnMouseDown, um den Focus von anderen Controls zu entfernen
    ''' </summary>
    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)
        Me.Focus()
    End Sub
#End Region

End Class
